--[[
Project name: HeadCount
Developed by: seppyk (http://www.authority-km.com)
Website: http://www.wowace.com/wiki/HeadCount
Description: Manages tracking of raid attendance and more.
License: LGPL v2.1
File: RaidListWrapper.lua
File description: Raid list wrapper object
]]

local AceOO = AceLibrary("AceOO-2.0")
local L = AceLibrary("AceLocale-2.2"):new("HeadCount")

HeadCount.ExclusionListWrapper = AceOO.Class()

HeadCount.ExclusionListWrapper.prototype.exclusionList = nil
HeadCount.ExclusionListWrapper.prototype.numberOfExcludedItems = nil

function HeadCount.ExclusionListWrapper.prototype:init(args)
    self.class.super.prototype.init(self)

	self.exclusionList = args["exclusionList"]
	self.numberOfExcludedItems = args["numberOfExcludedItems"]
	
	self.type = "HeadCountExclusionListWrapper-1.0"
end

-- Lists the exclusion list items
function HeadCount.ExclusionListWrapper.prototype:list() 
	HeadCount:LogInformation(string.format(L["info.exclude.loot.title"], L["product.name"], L["product.version.major"], L["product.version.minor"], self.numberOfExcludedItems))
	for k,v in pairs(self.exclusionList) do 
		HeadCount:LogInformation(string.format(L["info.exclude.loot"], L["product.name"], L["product.version.major"], L["product.version.minor"], v:getLink(), k))
	end
end

-- Adds an item.
-- @param id The loot id
-- @param loot The loot item..
function HeadCount.ExclusionListWrapper.prototype:addItem(id, loot)
	if (id and loot) then 
		self.exclusionList[id] = loot
		self.numberOfExcludedItems = self.numberOfExcludedItems + 1
	end
end

-- Remove an item.
-- @param id The item id.
-- @return boolean Returns true if the item was removed and false otherwise.
function HeadCount.ExclusionListWrapper.prototype:removeItem(id)
	local isRemoved = false

	if (self.exclusionList[id]) then
		-- the item exists
		self.exclusionList[id] = nil
		self.numberOfExcludedItems = self.numberOfExcludedItems - 1
		isRemoved = true
	end

	return isRemoved
end

-- Determines if an item is present.
-- @param item The item.
-- @return boolean Returns true if the item is present and false otherwise.
function HeadCount.ExclusionListWrapper.prototype:isItemPresent(item) 
	local isPresent = false
	
	if (self.exclusionList[item]) then
		isPresent = true
	end
	
	return isPresent
end

-- Retrieves the total number of items.
-- @return number Returns the total number of items.
function HeadCount.ExclusionListWrapper.prototype:getNumberOfExcludedItems() 
	return self.numberOfExcludedItems
end

-- Gets the exclusion list.
-- @return table Returns the exclusion list.			
function HeadCount.ExclusionListWrapper.prototype:getExclusionList() 
	return self.exclusionList
end

-- Sets the exclusion list.
-- @param exclusionList The exclusion list.
function HeadCount.ExclusionListWrapper.prototype:setExclusionList(exclusionList) 
	self.exclusionList = exclusionList
end

-- Serialization method.
function HeadCount.ExclusionListWrapper.prototype:Serialize() 
	local s = { }
	
	for k,v in pairs(self) do
		if type(v) ~= "function" and type(v) ~= "userdata" and k ~= "header" and k ~= "btnframe" and k ~= "temp" and k ~= "theme" and k ~= "base" and k ~= "curState" then
			s[k] = v
		end
	end
	
	return s
end

-- Deserialization method.
function HeadCount.ExclusionListWrapper:Deserialize(t) 
	return self:new(t)
end

-- To String
-- @return string Returns the string description for this object.
function HeadCount.ExclusionListWrapper.prototype:ToString()
	return L["object.ExclusionListWrapper"]
end

AceLibrary:Register(HeadCount.ExclusionListWrapper, "HeadCountExclusionListWrapper-1.0", 1)