--[[
Project name: HeadCount
Developed by: seppyk (http://www.authority-km.com)
Website: http://www.wowace.com/wiki/HeadCount
Description: Manages tracking of raid attendance and more.
License: LGPL v2.1
File: HeadCountFrames.lua
File description: GUI application logic
]]


local L = AceLibrary("AceLocale-2.2"):new("HeadCount")

local content = {
	["raid"] = "Raid members",
	["loot"] = "Raid loot", 
	["player"] = "Player", 
	["default"] = "Default", 
}

local confirmType = {
	["raid"] = "raid", 
	["member"] = "member", 
	["loot"] = "loot", 
	["endraid"] = "endraid", 
	["removeall"] = "removeall", 
}

-- ***********************************
-- MAIN FUNCTIONS
-- ***********************************
-- Main update process
function HeadCountFrame_Update()
	-- Update everything
	HeadCountFrameRaidHistoryContentScroll_Update()

	if (HeadCountFrame.contentType == content["raid"]) then
		HeadCountFrameContentMembersScroll_Update()		
		HeadCountFrameContentPlayer:Hide()
		HeadCountFrameContentLoot:Hide()		
		HeadCountFrameContentMembers:Show()		
	elseif (HeadCountFrame.contentType == content["loot"]) then 
		HeadCountFrameContentLootScroll_Update()		
		HeadCountFrameContentMembers:Hide()		
		HeadCountFrameContentPlayer:Hide()
		HeadCountFrameContentLoot:Show()		
	elseif (HeadCountFrame.contentType == content["player"]) then
		HeadCountFrameContentPlayerScroll_Update()		
		HeadCountFrameContentMembers:Hide()
		HeadCountFrameContentLoot:Hide()		
		HeadCountFrameContentPlayer:Show() 
	else
		-- default
		HeadCountFrameContentMembers:Hide()
		HeadCountFrameContentLoot:Hide()		
		HeadCountFrameContentPlayer:Hide() 		
		HeadCountFrameContentTitleString_Show()
	end
	
	-- TODO: Hide show appropriate content frames
end

-- Hide the user interface
function HideUserInterface()
	DisableModalFrame()
	
	-- close all frames	
	HeadCountFrame:Hide()	-- close the main frame
end

-- Show the user interface
function ShowUserInterface()
	-- show the main frame
	HeadCountFrame:Show()
end

-- Toggle the user interface
function ToggleUserInterface()
	if (HeadCountFrame:IsVisible()) then
		-- frame is visible, hide it
		HideUserInterface()
	else
		ShowUserInterface()
	end
end

-- Enable to modal frame (remove/export)
function EnableModalFrame() 
	HeadCountFrame.isDialogDisplayed = true		
end

-- Disable to modal frame (remove/export)
function DisableModalFrame() 
	HeadCountFrame.isDialogDisplayed = false
	HeadCountFrameConfirm:Hide()
	HeadCountFrameExport:Hide()
end


-- ***********************************
-- ON LOAD FUNCTIONS
-- ***********************************
-- ON LOAD: Confirm frame
function HeadCountFrameConfirm_Load()
	HeadCountFrameConfirmConfirmButton:SetText(L["remove"])
	HeadCountFrameConfirmCancelButton:SetText(L["cancel"]) 
end

-- ON LOAD: Export frame
function HeadCountFrameExport_Load() 
	HeadCountFrameExportCloseButton:SetText(L["Close"])
end

-- ON LOAD: Main frame
function HeadCountFrame_Load()
	HeadCountFrame.isRaidSelected = false
	HeadCountFrame.selectedRaidId = 0	
	HeadCountFrame.isDialogDisplayed = false
end

-- ON LOAD: Raid history frame
function HeadCountFrameRaidHistory_Load()
	HeadCountFrameRaidHistory:SetBackdropBorderColor(0.8, 0.8, 0.8, 0.8)
	HeadCountFrameRaidHistoryTitleString:SetText(string.format(L["info.raidhistory"], 0))
end

-- ON LOAD: Raid history content frame
function HeadCountFrameRaidHistoryContent_Load()
	--HeadCountFrameRaidHistoryContentScroll:Show()
end

-- ON LOAD: Content frame
function HeadCountFrameContent_Load() 
	HeadCountFrameContent:SetBackdropBorderColor(0.8, 0.8, 0.8, 0.8)
	HeadCountFrame.contentType = content["raid"]
	HeadCountFrameContentTitleString:Hide()
end

-- ON LOAD: Raid members
function HeadCountFrameContentMembers_Load()
	HeadCountFrameContentMembersNameButton:SetText(L["Name"])	
	HeadCountFrameContentMembersStartTimeButtonText:SetText(L["start"])	
	HeadCountFrameContentMembersEndTimeButtonText:SetText(L["end"])	
	HeadCountFrameContentMembersTotalTimeButtonText:SetText(L["Total time"])
	
	HeadCountFrameContentMembers.sortType = "Start"
	HeadCountFrameContentMembers.isDescending = true
end

-- ON LOAD: Player information
function HeadCountFrameContentPlayer_Load() 
	HeadCountFrameContentPlayerBackButtonText:SetText(L["Go back"])
end

-- ON LOAD: Title head
function HeadCountFrameTitleHeader_Load()
	HeadCountFrameTitleHeaderString:SetText(L["product.name"])
end

-- ON LOAD: Raid history template
function HeadCountFrameRaidHistoryContentTemplate_Load()
	getglobal(this:GetName() .. "MouseOver"):SetAlpha(0.3)
	getglobal(this:GetName() .. "MouseOver"):SetVertexColor(0.4, 0.4, 1.0)
				
	getglobal(this:GetName() .. "MouseSelect"):SetAlpha(0.6)
	getglobal(this:GetName() .. "MouseSelect"):SetVertexColor(0.4, 0.4, 1.0)
end

-- ON LOAD: Raid members template
function HeadCountFrameContentMembersTemplate_Load()
	getglobal(this:GetName() .. "MouseOver"):SetAlpha(0.3)
	getglobal(this:GetName() .. "MouseOver"):SetVertexColor(0.4, 0.4, 1.0)
end

-- ***********************************
-- SCROLL UPDATE FUNCTIONS
-- ***********************************
-- ON UPDATE: Raid history content frame scroller
function HeadCountFrameRaidHistoryContentScroll_Update()
	local lineNumber
	local lineNumberOffset
	--FauxScrollFrame_Update(FRAME, NUMBER_OF_ENTRIES, NUMBER_OF_ENTRIES_DISPLAYED, ENTRY_PIXEL_HEIGHT, button, smallWidth, bigWidtth, highlightFrame, smallHighlightWidth, bigHighlightWidth) 

	local raidTracker = HeadCount:getRaidTracker() 
	local numberOfRaids = raidTracker:getNumberOfRaids()
	FauxScrollFrame_Update(HeadCountFrameRaidHistoryContentScroll, numberOfRaids, 5, 16)
	
	if (numberOfRaids > 0) then	
		local orderedRaidList = raidTracker:retrieveOrderedRaidList(true)	
		
		for lineNumber=1,5 do
			lineNumberOffset = lineNumber + FauxScrollFrame_GetOffset(HeadCountFrameRaidHistoryContentScroll)
			if (lineNumberOffset <= numberOfRaids) then				
				getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber .. "MouseOver"):Hide()
				getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber .. "HitArea"):Show()

				-- Set the raid history line text
				local position = (numberOfRaids + 1) - lineNumberOffset			
				local raid = orderedRaidList[position]
				local raidId = raid:retrieveStartingTime():getUTCDateTimeInSeconds() 
				getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber .. "Id"):SetText(raidId)
				getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber .. "Number"):SetText(position)
				getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber .. "Time"):SetText(HeadCount:getDateTimeAsString(raid:retrieveStartingTime()))
				
				-- Display if the active raid text if it exists
				local zone = raid:getZone()
				if ((not raid:getIsFinalized()) and (raidTracker:getIsCurrentRaidActive())) then 
					-- raid is not finalized and raid is active
					if (zone) then
						zone = zone .. " (" .. L["active"] .. ")"
					else
						zone = "(" .. L["active"] .. ")"
					end
				end
				
				getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber .. "Zone"):SetText(zone)				
				getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber):Show()
				
				-- Set the selected color bar if this raid is selected
				if (HeadCountFrame.selectedRaidId == raidId) then
					-- This line is equivalent to the currently selected raid id
					getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber .. "MouseSelect"):Show()
				else
					-- This line is not equivalent to the currently selected raid id, don't select it
					getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber .. "MouseSelect"):Hide()
				end
			else
				getglobal("HeadCountFrameRaidHistoryContentEntry" .. lineNumber):Hide()
			end
		end
		
		getglobal("HeadCountFrameRaidHistoryContent"):Show()
	else
		-- No raids exist
		getglobal("HeadCountFrameRaidHistoryContent"):Hide()	-- hide the raid history content frame
	end
	
	HeadCountFrameRaidHistoryTitleString:SetText(string.format(L["info.raidhistory"], numberOfRaids))	-- update the amount of raids text string	
end

-- ON UPDATE: Members content frame scroller
function HeadCountFrameContentMembersScroll_Update()
	local lineNumber
	local lineNumberOffset
	local numberOfPlayers = 0
	
	local raidId = HeadCountFrame.selectedRaidId
	local raidTracker = HeadCount:getRaidTracker()
	local raid = raidTracker:getRaidById(raidId)
	
	if (raid) then
		-- raid exists
		numberOfPlayers = raid:getNumberOfPlayers()
		FauxScrollFrame_Update(HeadCountFrameContentMembersScroll, numberOfPlayers, 10, 16)	                                     
		
		if (numberOfPlayers > 0) then	
			-- players exist in this raid
			local orderedPlayerList = raid:retrieveOrderedPlayerList(HeadCount:getRAID_MEMBER_SORT()[HeadCountFrameContentMembers.sortType], HeadCountFrameContentMembers.isDescending)		

			for lineNumber=1,10 do
				lineNumberOffset = lineNumber + FauxScrollFrame_GetOffset(HeadCountFrameContentMembersScroll)
				if (lineNumberOffset <= numberOfPlayers) then				
					getglobal("HeadCountFrameContentMembersEntry" .. lineNumber .. "MouseOver"):Hide()
					getglobal("HeadCountFrameContentMembersEntry" .. lineNumber .. "HitArea"):Show()
					local player = orderedPlayerList[lineNumberOffset]
					
					getglobal("HeadCountFrameContentMembersEntry" .. lineNumber .. "Number"):SetText(lineNumberOffset)
					getglobal("HeadCountFrameContentMembersEntry" .. lineNumber .. "Name"):SetText(player:getName())
					getglobal("HeadCountFrameContentMembersEntry" .. lineNumber .. "BeginTime"):SetText(HeadCount:getTimeAsString(player:retrieveStartingTime()))
					
					if (player:retrieveEndingTime()) then
						-- end time exists
						getglobal("HeadCountFrameContentMembersEntry" .. lineNumber .. "EndTime"):SetText(HeadCount:getTimeAsString(player:retrieveEndingTime()))
					else
						-- end time not yet set, show nothing
						getglobal("HeadCountFrameContentMembersEntry" .. lineNumber .. "EndTime"):SetText("")
					end
					
					if (raid:getIsFinalized()) then
						-- raid is finalized, show total time
						getglobal("HeadCountFrameContentMembersEntry" .. lineNumber .. "TotalTime"):SetText(HeadCount:getSecondsAsString(player:getTotalTime()))
					else
						-- raid is not finalized show nothing
						getglobal("HeadCountFrameContentMembersEntry" .. lineNumber .. "TotalTime"):SetText("")
					end
					
					getglobal("HeadCountFrameContentMembersEntry" .. lineNumber):Show()
				else
					getglobal("HeadCountFrameContentMembersEntry" .. lineNumber):Hide()
				end
			end
			HeadCountFrameContentMembers_Show(true)
		else
			-- players do not exist
			for lineNumber=1,10 do 
				getglobal("HeadCountFrameContentMembersEntry" .. lineNumber):Hide()
			end
			
			HeadCountFrameContentMembers_Show(false)	-- hide the members content frame
		end		
	else
		-- no raid exists under this id
		HeadCountFrame.isRaidSelected = false
		HeadCountFrame.selectedRaidId = 0			
		HeadCountFrameContentMembers_Show(false)		-- hide the members content frame
	end
	
	HeadCountFrameContentTitleString_Show()
end

-- ON UPDATE: Loot content frame scroller
function HeadCountFrameContentLootScroll_Update() 
	local lineNumber
	local lineNumberOffset
	
	local raidId = HeadCountFrame.selectedRaidId
	local raidTracker = HeadCount:getRaidTracker()
	local selectedRaid = raidTracker:getRaidById(raidId)
	
	if (selectedRaid) then 
		-- raid exists 
		local numberOfLoots = selectedRaid:numberOfLoots() 
		FauxScrollFrame_Update(HeadCountFrameContentLootScroll, numberOfLoots, 5, 40)	 
		
		if (numberOfLoots > 0) then 
			local lootList = selectedRaid:getLootList() 	
						
			for lineNumber=1,5 do
				lineNumberOffset = lineNumber + FauxScrollFrame_GetOffset(HeadCountFrameContentLootScroll)
				if (lineNumberOffset <= numberOfLoots) then 
					local loot = lootList[lineNumberOffset]
					
					getglobal("HeadCountFrameContentLootEntry" .. lineNumber .. "TextureButton"):SetNormalTexture(loot:getTexture())
					getglobal("HeadCountFrameContentLootEntry" .. lineNumber .. "Number"):SetText(lineNumberOffset)
					getglobal("HeadCountFrameContentLootEntry" .. lineNumber .. "Name"):SetText(loot:getLink()) 																								 
					getglobal("HeadCountFrameContentLootEntry" .. lineNumber .. "Looter"):SetText(L["Looted by"] .. " " .. loot:getPlayerName() .. " (" .. HeadCount:getDateTimeAsString(loot:getActivityTime())  .. ")")
				
					getglobal("HeadCountFrameContentLootEntry" .. lineNumber):Show()
				else
					getglobal("HeadCountFrameContentLootEntry" .. lineNumber):Hide()
				end
			end
			HeadCountFrameContentLoot_Show(true)
		else
			for lineNumber=1,5 do 
				getglobal("HeadCountFrameContentLootEntry" .. lineNumber):Hide()
			end
			HeadCountFrameContentLoot_Show(false)
		end		
	else
		-- no raid exists under this id 
		HeadCountFrame.isRaidSelected = false
		HeadCountFrame.selectedRaidId = 0
		HeadCountFrameContentLoot_Show(false)
	end
	
	HeadCountFrameContentTitleString_Show()
end

-- ON UPDATE: Player content frame scroller
function HeadCountFrameContentPlayerScroll_Update() 
	local lineNumber
	local lineNumberOffset

	local raidId = HeadCountFrame.selectedRaidId
	local raidTracker = HeadCount:getRaidTracker()
	local selectedRaid = raidTracker:getRaidById(raidId)

	if (selectedRaid) then 
		-- raid exists
		local player = selectedRaid:retrievePlayer(HeadCountFrame.playerName)		
		if (player) then 
			local playerInformation, numberOfPlayerInformationLines = player:retrieveInformationList()
		
			FauxScrollFrame_Update(HeadCountFrameContentPlayerScroll, numberOfPlayerInformationLines, 10, 16)		
		
			for lineNumber=1,10 do
				lineNumberOffset = lineNumber + FauxScrollFrame_GetOffset(HeadCountFrameContentPlayerScroll)
				if (lineNumberOffset <= numberOfPlayerInformationLines) then			
					getglobal("HeadCountFrameContentPlayerEntry" .. lineNumber .. "Information"):SetText(playerInformation[lineNumberOffset])			
					getglobal("HeadCountFrameContentPlayerEntry" .. lineNumber):Show()
				else
					getglobal("HeadCountFrameContentPlayerEntry" .. lineNumber):Hide()
				end
			end
			
			HeadCountFrameContentPlayerNameString:SetText(player:getName())
			HeadCountFrameContentPlayer_Show(true)			
		else
			-- player does not exist
			HeadCountFrameContentPlayer_Show(false)		
		end		
	else
		-- raid does not exist
		HeadCountFrameContentPlayer_Show(false)
	end
	
	HeadCountFrameContentTitleString_Show()
end

-- ***********************************
-- SHOW FUNCTIONS
-- ***********************************
-- SHOW: Toggle display of members content frame
function HeadCountFrameContentMembers_Show(isShowing)
	if (isShowing) then
		getglobal("HeadCountFrameContentMembers"):Show()	
	else
		getglobal("HeadCountFrameContentMembers"):Hide()	
	end
end

-- SHOW: Toggle display of loot content frame
function HeadCountFrameContentLoot_Show(isShowing)  
	if (isShowing) then 
		getglobal("HeadCountFrameContentLoot"):Show()
	else
		getglobal("HeadCountFrameContentLoot"):Hide()
	end
end

-- SHOW: Toggle display of player content frame
function HeadCountFrameContentPlayer_Show(isShowing) 
	if (isShowing) then
		getglobal("HeadCountFrameContentPlayer"):Show()	
	else
		getglobal("HeadCountFrameContentPlayer"):Hide()	
	end
end

-- SHOW: Display the correct content title string
function HeadCountFrameContentTitleString_Show()
	local titleString

	local raidTracker = HeadCount:getRaidTracker()
	local numberOfRaids = raidTracker:getNumberOfRaids()

	if (numberOfRaids > 0) then
		if (HeadCountFrame.isRaidSelected) then 
			local raid = raidTracker:getRaidById(HeadCountFrame.selectedRaidId)
			
			if (content["raid"] == HeadCountFrame.contentType) then
				local numberOfPlayers = raid:getNumberOfPlayers()
				
				titleString = string.format(L["info.raidmembers"], numberOfPlayers)

				HeadCountFrameContentTitleString:Show()
				HeadCountFrameContentTitleString:SetText(titleString)			
				HeadCountFrameContentMembersButton:Show()
				HeadCountFrameContentLootButton:Show()		
			elseif (content["loot"] == HeadCountFrame.contentType) then
				local numberOfLoots = raid:numberOfLoots() 
				
				titleString = string.format(L["info.raidloot"], numberOfLoots) 
				
				HeadCountFrameContentTitleString:Show()
				HeadCountFrameContentTitleString:SetText(titleString)
				HeadCountFrameContentMembersButton:Show()
				HeadCountFrameContentLootButton:Show()									
			elseif (content["player"] == HeadCountFrame.contentType) then 
				titleString = L["info.raidplayer"]
				
				HeadCountFrameContentTitleString:Show()
				HeadCountFrameContentTitleString:SetText(titleString)			
				HeadCountFrameContentMembersButton:Show()
				HeadCountFrameContentLootButton:Show()					
			else
				HeadCountFrameContentTitleString:Hide()
			end	
		else
			-- raids exist, but none are selected
			titleString = L["info.noraidsselected"]
			HeadCountFrameContentTitleString:Show()
			HeadCountFrameContentTitleString:SetText(titleString)		
			HeadCountFrameContentMembersButton:Hide()
			HeadCountFrameContentLootButton:Hide()		
		end
	else
		titleString = L["info.noraidsexist"]
		HeadCountFrameContentTitleString:Show()
		HeadCountFrameContentTitleString:SetText(titleString)		
		HeadCountFrameContentMembersButton:Hide()
		HeadCountFrameContentLootButton:Hide()		
	end
end

-- SHOW: Display the confirm frame
-- @param frameType The frame type
-- @param raidId The raid id
function HeadCountFrameConfirm_Show(frameType, raidId) 
	if (not HeadCountFrame.isDialogDisplayed) then
		-- only allow display of the remove frame is it is current not being displayed	
		if (confirmType[frameType]) then
			-- removal type is valid
			local isDisplayingConfirmFrame = false								
			local raidTracker = HeadCount:getRaidTracker()
			
			if (confirmType["raid"] == frameType) then
				-- remove raid
				HeadCountFrameConfirm.frameType = confirmType["raid"]
				HeadCountFrameConfirmHeaderTitleString:SetText(L["Remove raid"])		
				HeadCountFrameConfirmInfo:SetText(L["info.remove.raid"])						
				HeadCountFrameConfirmConfirmButton:SetText(L["remove"])
				HeadCountFrameConfirmCancelButton:SetText(L["cancel"])				

				local selectedRaid = raidTracker:getRaidById(raidId)
				local zone = selectedRaid:getZone()
				
				HeadCountFrameConfirm.description = HeadCount:getDateTimeAsString(selectedRaid:retrieveStartingTime())
				if (zone) then
					HeadCountFrameConfirm.description = HeadCountFrameConfirm.description .. " - " .. zone
				end
					
				isDisplayingConfirmFrame = true
			elseif (confirmType["member"] == frameType) then
				-- remove member
				HeadCountFrameConfirm.frameType = confirmType["member"]
				HeadCountFrameConfirmHeaderTitleString:SetText(L["Remove member"])
				HeadCountFrameConfirmInfo:SetText(L["info.remove.member"])		
				HeadCountFrameConfirmConfirmButton:SetText(L["remove"])
				HeadCountFrameConfirmCancelButton:SetText(L["cancel"])				
				
				isDisplayingConfirmFrame = true
			elseif (confirmType["endraid"] == frameType) then
				-- end the active raid
				if (raidTracker:isCurrentRaidEndable()) then
					local currentRaid = raidTracker:retrieveMostRecentRaid()
					local zone = currentRaid:getZone()
					
					HeadCountFrameConfirm.description = HeadCount:getDateTimeAsString(currentRaid:retrieveStartingTime())
					if (zone) then
						HeadCountFrameConfirm.description = HeadCountFrameConfirm.description .. " - " .. zone
					end
					
					HeadCountFrameConfirm.frameType = confirmType["endraid"]
					HeadCountFrameConfirmHeaderTitleString:SetText(L["End raid"])
					HeadCountFrameConfirmInfo:SetText(L["info.end.raid"])
					HeadCountFrameConfirmConfirmButton:SetText(L["End raid"])
					HeadCountFrameConfirmCancelButton:SetText(L["cancel"])

					isDisplayingConfirmFrame = true	
				else
					HeadCount:LogInformation(string.format(L["info.end.raid.noraidsexists"], L["product.name"], L["product.version.major"], L["product.version.minor"]))
				end
			elseif (confirmType["removeall"] == frameType) then 
				-- remove all raids
				HeadCountFrameConfirm.frameType = confirmType["removeall"]
				HeadCountFrameConfirmHeaderTitleString:SetText(L["Remove all"])
				HeadCountFrameConfirmInfo:SetText(L["info.remove.allraids"])
				HeadCountFrameConfirmConfirmButton:SetText(L["remove"])
				HeadCountFrameConfirmCancelButton:SetText(L["cancel"])				
				HeadCountFrameConfirm.description = L["info.remove.allraids.warning"]
				
				isDisplayingConfirmFrame = true				
			elseif (confirmType["loot"] == frameType) then 
				-- remove loot
				HeadCountFrameConfirm.frameType = confirmType["loot"]
				HeadCountFrameConfirmHeaderTitleString:SetText(L["Remove loot"])
				HeadCountFrameConfirmInfo:SetText(L["info.remove.loot"])		
				HeadCountFrameConfirmConfirmButton:SetText(L["remove"])
				HeadCountFrameConfirmCancelButton:SetText(L["cancel"])				
				
				isDisplayingConfirmFrame = true
			end				

			if (isDisplayingConfirmFrame) then
				-- display the frame
				HeadCountFrameConfirm.raidId = raidId
				HeadCountFrameConfirmDescription:SetText(HeadCountFrameConfirm.description)						
			
				EnableModalFrame() 
				HeadCountFrameConfirm:Show()
			else
				-- do not display (hide) the frame
				DisableModalFrame() 		
			end
		end
	end
end

-- SHOW: Display the export frame
-- @param raidId The raid id
function HeadCountFrameExport_Show(raidId) 
	if (not HeadCountFrame.isDialogDisplayed) then
		-- only allow display of the remove frame is it is current not being displayed	
		local raidTracker = HeadCount:getRaidTracker()
		local raid = raidTracker:getRaidById(raidId)
		
		local exportString = raid:exportEQdkp()
		
		HeadCountFrameExportHeaderTitleString:SetText(L["Export raid"])
		HeadCountFrameExportInfo:SetText(string.format(L["info.export.raid"], HeadCount:GetExportFormat()))
		
		HeadCountFrameExportDescription:SetText(HeadCount:getDateTimeAsString(raid:retrieveStartingTime()))
		HeadCountFrameExportScrollContent:SetText(exportString)
		
		EnableModalFrame() 
		HeadCountFrameExport:Show()		
	end
end

-- ***********************************
-- ON CLICK FUNCTIONS
-- ***********************************
-- ON CLICK: Frame close button
function HeadCountFrameTitleCloseButton_Click()
	-- Main close button has been clicked
	HideUserInterface()
end

-- ON CLICK: End raid button
function HeadCountFrameRaidHistoryEndRaidButton_Click()
	local currentRaidId = HeadCount:getRaidTracker():retrieveCurrentRaidId()
	
	HeadCountFrameConfirm.description = ""
	HeadCountFrameConfirm_Show("endraid", currentRaidId)
end

-- ON CLICK: Remove all raids button
function HeadCountFrameRaidHistoryRemoveAllButton_Click() 
	HeadCountFrameConfirm.description = ""
	HeadCountFrameConfirm_Show("removeall", 0)
end

-- ON CLICK: Raid selected
function HeadCountFrameRaidHistoryContentTemplateHitArea_Click()
	local raidId = tonumber(getglobal(this:GetParent():GetName() .. "Id"):GetText())	-- save the selected raid id

	DisableModalFrame()
	
	HeadCountFrame.selectedRaidId = raidId 
	HeadCountFrame.isRaidSelected = true
	HeadCountFrame.contentType = content["raid"]	
	HeadCountFrame_Update()
end

-- ON CLICK: Raid export button
function HeadCountFrameRaidHistoryContentTemplateExportButton_Click() 
	local raidTracker = HeadCount:getRaidTracker() 
	local currentRaidId = raidTracker:retrieveCurrentRaidId()
	local isCurrentRaidActive = raidTracker:getIsCurrentRaidActive()
	
	local raidId = tonumber(getglobal(this:GetParent():GetName() .. "Id"):GetText())	

	if ((raidId == currentRaidId) and (isCurrentRaidActive)) then 
		-- user chose most recent raid and most recent raid is active
		HeadCount:LogInformation(string.format(L["info.export.raid.currentraid"], L["product.name"], L["product.version.major"], L["product.version.minor"]))	
	else
		-- user did not choose the most recent raid or the most recent raid is NOT active
		HeadCountFrameExport_Show(raidId)		
	end
end

-- ON CLICK: Raid delete button	
function HeadCountFrameRaidHistoryContentTemplateDeleteButton_Click()
	local raidId = tonumber(getglobal(this:GetParent():GetName() .. "Id"):GetText())
	HeadCountFrameConfirm_Show("raid", raidId)
end

-- ON CLICK: Raid members button
function HeadCountFrameContentMembersButton_Click() 
	DisableModalFrame() 
	
	HeadCountFrame.contentType = content["raid"]
	HeadCountFrame_Update()
end

-- ON CLICK: Raid loot button
function HeadCountFrameContentLootButton_Click() 
	DisableModalFrame()
	
	HeadCountFrame.contentType = content["loot"]
	HeadCountFrame_Update()
end

-- ON CLICK: Player go back button
function HeadCountFrameContentPlayerBackButton_Click() 
	DisableModalFrame() 
	
	HeadCountFrame.contentType = content["raid"]
	HeadCountFrame_Update()
end

-- ON CLICK: Raid member name sort button
function HeadCountFrameContentMembersNameButton_Click()
	local raidMemberSort = HeadCount:getRAID_MEMBER_SORT()
	
	if (raidMemberSort["Name"] == HeadCountFrameContentMembers.sortType) then
		-- Current column is sort column, switch direction
		HeadCountFrameContentMembers.isDescending = not HeadCountFrameContentMembers.isDescending
	else
		HeadCountFrameContentMembers.sortType = raidMemberSort["Name"]
		HeadCountFrameContentMembers.isDescending = true
	end	
	
	HeadCountFrame_Update()	
end

-- ON CLICK: Raid member start time sort button
function HeadCountFrameContentMembersStartTimeButton_Click() 
	local raidMemberSort = HeadCount:getRAID_MEMBER_SORT()
	
	if (raidMemberSort["Start"] == HeadCountFrameContentMembers.sortType) then
		-- Current column is sort column, switch direction
		HeadCountFrameContentMembers.isDescending = not HeadCountFrameContentMembers.isDescending
	else
		HeadCountFrameContentMembers.sortType = raidMemberSort["Start"]
		HeadCountFrameContentMembers.isDescending = true
	end	
	
	HeadCountFrame_Update()	
end

-- ON CLICK: Raid member end time sort button
function HeadCountFrameContentMembersEndTimeButton_Click() 
	local raidMemberSort = HeadCount:getRAID_MEMBER_SORT()
	
	if (raidMemberSort["End"] == HeadCountFrameContentMembers.sortType) then
		-- Current column is sort column, switch direction
		HeadCountFrameContentMembers.isDescending = not HeadCountFrameContentMembers.isDescending
	else
		HeadCountFrameContentMembers.sortType = raidMemberSort["End"]
		HeadCountFrameContentMembers.isDescending = true
	end	
	
	HeadCountFrame_Update()	
end

-- ON CLICK: Raid member total time sort button
function HeadCountFrameContentMembersTotalTimeButton_Click() 
	local raidMemberSort = HeadCount:getRAID_MEMBER_SORT()
	
	if (raidMemberSort["Total"] == HeadCountFrameContentMembers.sortType) then
		-- Current column is sort column, switch direction
		HeadCountFrameContentMembers.isDescending = not HeadCountFrameContentMembers.isDescending
	else
		HeadCountFrameContentMembers.sortType = raidMemberSort["Total"]
		HeadCountFrameContentMembers.isDescending = true
	end	
	
	HeadCountFrame_Update()
end

-- ON CLICK: Member selected
function HeadCountFrameContentMembersTemplateHitArea_Click() 
	DisableModalFrame()
	
	HeadCountFrame.playerName = getglobal(this:GetParent():GetName() .. "Name"):GetText()
	HeadCountFrame.contentType = content["player"]
	HeadCountFrame_Update()
end

-- ON CLICK: Member delete button
function HeadCountFrameContentMembersTemplateDeleteButton_Click()
	HeadCountFrameConfirm.playerId = tonumber(getglobal(this:GetParent():GetName() .. "Number"):GetText())
	HeadCountFrameConfirm.description = getglobal(this:GetParent():GetName() .. "Name"):GetText()
		
	HeadCountFrameConfirm_Show("member", HeadCountFrame.selectedRaidId)
end

-- ON CLICK: Loot delete button
function HeadCountFrameContentLootTemplateDeleteButton_Click() 
	HeadCountFrameConfirm.lootId = tonumber(getglobal(this:GetParent():GetName() .. "Number"):GetText())
	HeadCountFrameConfirm.description = getglobal(this:GetParent():GetName() .. "Name"):GetText()
	
	HeadCountFrameConfirm_Show("loot", HeadCountFrame.selectedRaidId)
end

-- ON CLICK: Confirm button
function HeadCountFrameConfirmConfirmButton_Click()		
	DisableModalFrame() 
	
	if (confirmType["raid"] == HeadCountFrameConfirm.frameType) then
		HeadCount:removeRaid(HeadCountFrameConfirm.raidId)
	elseif (confirmType["member"] == HeadCountFrameConfirm.frameType) then		                                               
		HeadCount:removePlayer(HeadCountFrameConfirm.raidId, HeadCountFrameConfirm.description)
	elseif (confirmType["loot"] == HeadCountFrameConfirm.frameType) then 
		HeadCount:removeLoot(HeadCountFrameConfirm.raidId, HeadCountFrameConfirm.lootId)	
	elseif (confirmType["endraid"] == HeadCountFrameConfirm.frameType) then 
		local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()
		local activityTime = AceLibrary("HeadCountTime-1.0"):new({ ["utcDateTimeInSeconds"] = utcDateTimeInSeconds })	
		HeadCount:endCurrentRaid(activityTime)
	elseif (confirmType["removeall"] == HeadCountFrameConfirm.frameType) then 
		HeadCountFrame.isRaidSelected = false
		HeadCountFrame.selectedRaidId = 0		
		HeadCountFrame.contentType = content["default"]
		HeadCount:removeAllRaids()
	end
end

-- ON CLICK: Cancel button
function HeadCountFrameConfirmCancelButton_Click()		
	DisableModalFrame() 
end

-- ON CLICK: Export close button
function HeadCountFrameExportCloseButton_Click()	
	DisableModalFrame() 
end

-- ***********************************
-- ON ENTER FUNCTIONS
-- ***********************************
-- ON ENTER: End active raid button
function HeadCountFrameRaidHistoryEndRaidButton_Enter()
	GameTooltip:SetOwner(getglobal(this:GetName()), "ANCHOR_RIGHT", -6, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["End active raid"], 0.63, 0.63, 1.0)
	GameTooltip:Show()
end
	
-- ON ENTER: Remove all raids button
function HeadCountFrameRaidHistoryRemoveAllButton_Enter() 
	GameTooltip:SetOwner(getglobal(this:GetName()), "ANCHOR_RIGHT", -6, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["Remove all raids"], 0.63, 0.63, 1.0)
	GameTooltip:Show()
end
	
-- ON ENTER: Raid export button				
function HeadCountFrameRaidHistoryContentTemplateExportButton_Enter()
	local raidId = getglobal(this:GetParent():GetName() .. "Number"):GetText()
	local raidTime = getglobal(this:GetParent():GetName() .. "Time"):GetText()
	local zone = getglobal(this:GetParent():GetName() .. "Zone"):GetText()

	GameTooltip:SetOwner(this, "ANCHOR_RIGHT", -6, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["Export raid"] .. " " .. raidId .. ": ", 0.63, 0.63, 1.0)
	GameTooltip:AddLine(raidTime, 1.0, 1.0, 1.0)
	GameTooltip:AddLine(zone, 1.0, 1.0, 1.0)
	GameTooltip:Show()
						
	getglobal(this:GetParent():GetName() .. "MouseOver"):Show()
end
	
-- ON ENTER: Raid is selected
function HeadCountFrameRaidHistoryContentTemplateHitArea_Enter()
	getglobal(this:GetParent():GetName() .. "MouseOver"):Show()
end
	
-- ON ENTER: Raid delete button
function HeadCountFrameRaidHistoryContentTemplateDeleteButton_Enter()
	local raidNumber = getglobal(this:GetParent():GetName() .. "Number"):GetText()
	local raidTime = getglobal(this:GetParent():GetName() .. "Time"):GetText()
	local zone = getglobal(this:GetParent():GetName() .. "Zone"):GetText()

	GameTooltip:SetOwner(this, "ANCHOR_RIGHT", -6, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["Remove raid"] .. " " .. raidNumber .. ": ", 0.63, 0.63, 1.0)
	GameTooltip:AddLine(raidTime, 1.0, 1.0, 1.0)
	GameTooltip:AddLine(zone, 1.0, 1.0, 1.0)
	GameTooltip:Show()
	
	getglobal(this:GetParent():GetName() .. "MouseOver"):Show()
end				
	
-- ON ENTER: Raid members button
function HeadCountFrameContentMembersButton_Enter() 
	local raidTracker = HeadCount:getRaidTracker()	
	local selectedRaid = raidTracker:getRaidById(HeadCountFrame.selectedRaidId)
	local raidZone = selectedRaid:getZone()

	GameTooltip:SetOwner(this, "ANCHOR_RIGHT", -6, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["View raid members"], 0.63, 0.63, 1.0)
	GameTooltip:AddLine(HeadCount:getDateTimeAsString(selectedRaid:retrieveStartingTime()), 1.0, 1.0, 1.0)
	
	if (raidZone) then
		GameTooltip:AddLine(raidZone, 1.0, 1.0, 1.0)
	end
	
	GameTooltip:AddLine(L["Number of members"] .. ": " .. selectedRaid:getNumberOfPlayers(), 1.0, 1.0, 1.0)	
	
	GameTooltip:Show()
end
	
-- ON ENTER: Raid loot button
function HeadCountFrameContentLootButton_Enter() 
	local raidTracker = HeadCount:getRaidTracker()	
	local selectedRaid = raidTracker:getRaidById(HeadCountFrame.selectedRaidId)
	local raidZone = selectedRaid:getZone()
	
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT", -6, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["View raid loot"], 0.63, 0.63, 1.0)
	GameTooltip:AddLine(HeadCount:getDateTimeAsString(selectedRaid:retrieveStartingTime()), 1.0, 1.0, 1.0)
	GameTooltip:AddLine(L["Number of items"] .. ": " .. selectedRaid:numberOfLoots(), 1.0, 1.0, 1.0)	
	
	if (raidZone) then
		GameTooltip:AddLine(raidZone, 1.0, 1.0, 1.0)
	end
	
	GameTooltip:Show()
end
	
-- ON ENTER: Raid member selection
function HeadCountFrameContentMembersTemplateHitArea_Enter()
	getglobal(this:GetParent():GetName() .. "MouseOver"):Show()
end
	
-- ON ENTER: Raid member delete button
function HeadCountFrameContentMembersTemplateDeleteButton_Enter()
	local raidTracker = HeadCount:getRaidTracker()	
	local selectedRaid = raidTracker:getRaidById(HeadCountFrame.selectedRaidId)
	local raidZone = selectedRaid:getZone()
	
	local playerId = getglobal(this:GetParent():GetName() .. "Number"):GetText()
	local playerName = getglobal(this:GetParent():GetName() .. "Name"):GetText()

	GameTooltip:SetOwner(this, "ANCHOR_RIGHT", -6, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["Remove member"] .. " " .. playerId .. ": ", 0.63, 0.63, 1.0)
	GameTooltip:AddLine(playerName, 1.0, 1.0, 1.0)
	GameTooltip:AddLine(HeadCount:getDateTimeAsString(selectedRaid:retrieveStartingTime()), 1.0, 1.0, 1.0)
	
	if (raidZone) then
		GameTooltip:AddLine(raidZone, 1.0, 1.0, 1.0)
	end
	
	GameTooltip:Show()
						
	getglobal(this:GetParent():GetName() .. "MouseOver"):Show()
end
	
-- ON ENTER: Raid loot texture button.
function HeadCountFrameContentLootTemplateTextureButton_Enter() 
	local link = getglobal(this:GetParent():GetName() .. "Name"):GetText()
	
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT", -6, 0)
	GameTooltip:ClearLines()
	GameTooltip:SetHyperlink(link)
	GameTooltip:Show()
end	

-- ON ENTER: Raid loot delete button	
function HeadCountFrameContentLootTemplateDeleteButton_Enter() 
	local raidTracker = HeadCount:getRaidTracker()	
	local selectedRaid = raidTracker:getRaidById(HeadCountFrame.selectedRaidId)
	local raidZone = selectedRaid:getZone()
	
	local lootId = getglobal(this:GetParent():GetName() .. "Number"):GetText()
	local lootName = getglobal(this:GetParent():GetName() .. "Name"):GetText()
	
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT", -6, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["Remove loot"] .. " " .. lootId .. ": ", 0.63, 0.63, 1.0)
	GameTooltip:AddLine(lootName)
	
	GameTooltip:Show()
end
	
-- ***********************************
-- ON LEAVE FUNCTIONS
-- ***********************************
-- ON LEAVE: End active raid button
function HeadCountFrameRaidHistoryEndRaidButton_Leave()
	GameTooltip:Hide()
end

-- ON LEAVE: Remove all raids button
function HeadCountFrameRaidHistoryRemoveAllButton_Leave() 
	GameTooltip:Hide()
end

-- ON LEAVE: Raid is selected	
function HeadCountFrameRaidHistoryContentTemplateHitArea_Leave()
	getglobal(this:GetParent():GetName() .. "MouseOver"):Hide()
end

-- ON LEAVE: Raid export button
function HeadCountFrameRaidHistoryContentTemplateExportButton_Leave()
	getglobal(this:GetParent():GetName() .. "MouseOver"):Hide()
	GameTooltip:Hide()
end

-- ON LEAVE: Raid delete button		
function HeadCountFrameRaidHistoryContentTemplateDeleteButton_Leave()
	getglobal(this:GetParent():GetName() .. "MouseOver"):Hide()
	GameTooltip:Hide()
end

-- ON LEAVE: Raid members button
function HeadCountFrameContentMembersButton_Leave() 
	GameTooltip:Hide()
end

-- ON LEAVE: Raid loot button
function HeadCountFrameContentLootButton_Leave() 
	GameTooltip:Hide()
end

-- ON LEAVE: Raid member selection
function HeadCountFrameContentMembersTemplateHitArea_Leave()
	getglobal(this:GetParent():GetName() .. "MouseOver"):Hide();
end

-- ON LEAVE: Raid member delete button
function HeadCountFrameContentMembersTemplateDeleteButton_Leave()
	getglobal(this:GetParent():GetName() .. "MouseOver"):Hide()
	GameTooltip:Hide()
end			

-- ON LEAVE: Raid loot texture button.
function HeadCountFrameContentLootTemplateTextureButton_Leave() 
	GameTooltip:Hide()
end

-- ON LEAVE: Raid loot delete button.
function HeadCountFrameContentLootTemplateDeleteButton_Leave() 	
	GameTooltip:Hide()
end
