--[[
Project name: HeadCount
Developed by: seppyk (http://www.authority-km.com)
Website: http://www.wowace.com/wiki/HeadCount
Description: Manages tracking of raid attendance and more.
License: LGPL v2.1
File: HeadCountOptions.lua
File description: Options listing
]]

local L = AceLibrary("AceLocale-2.2"):new("HeadCount")

-- configuration options
local options = { 
    type = "group", 
	handler = HeadCount, 
    args = {
        gui = {
            type = "execute", 
            name = L["console.command.gui.name"],
            desc = L["console.command.gui.description"],
			func = "ToggleUI", 
			order = 1, 
        },	
		attendance = {
			type = "group", 
			name = L["console.command.attendance.name"], 
			desc = L["console.command.attendance.description"], 
			order = 2, 
		    args = {
				raidlistgroups = { 
					type = "text", 
					name = L["console.command.attendance.raidlistgroups.name"], 
					desc = L["console.command.attendance.raidlistgroups.description"], 
					usage = L["console.usage.groups"], 
					multiToggle = true, 
					get = "GetRaidListGroups", 
					set = "SetRaidListGroups", 
					validate = L.partyGroups, 
					order = 1, 					
				}, 
				waitlistgroups = { 
					type = "text", 
					name = L["console.command.attendance.waitlistgroups.name"], 
					desc = L["console.command.attendance.waitlistgroups.description"], 
					usage = L["console.usage.groups"], 
					multiToggle = true, 
					get = "GetWaitListGroups", 
					set = "SetWaitListGroups",
					validate = L.partyGroups, 					
					order = 2, 					
				}, 
			}, 
		}, 
		["time"] = {
			type = "group", 
			name = L["console.command.time.name"], 
			desc = L["console.command.time.description"], 
			order = 3, 
			args = {
				timetotals = { 
					type = "group", 
					name = L["console.command.time.timetotals.name"], 
					desc = L["console.command.time.timetotals.description"], 
					order = 1, 
					args = {
						raidlisttime = {
							type = "toggle", 
							name = L["console.command.time.timetotals.raidlisttime.name"], 
							desc = L["console.command.time.timetotals.raidlisttime.description"], 
							get = "IsRaidListTimeEnabled", 
							set = "ToggleRaidListTime", 
							order = 1, 
						}, 
						waitlisttime = {
							type = "toggle", 
							name = L["console.command.time.timetotals.waitlisttime.name"], 
							desc = L["console.command.time.timetotals.waitlisttime.description"], 							
							get = "IsWaitListTimeEnabled", 
							set = "ToggleWaitListTime", 							
							order = 2, 
						}, 
						offlinetime = {
							type = "toggle", 
							name = L["console.command.time.timetotals.offlinetime.name"], 
							desc = L["console.command.time.timetotals.offlinetime.description"], 							
							get = "IsOfflineTimeEnabled", 
							set = "ToggleOfflineTime", 
							order = 3, 
						}, 
					}, 
				}, 
				timezone = { 
					type = "range", 
					name = L["console.command.time.timezone.name"], 
					desc = L["console.command.time.timezone.description"], 
					get = "GetTimezone", 
					set = "SetTimezone", 
					min = -10 ,
					max = 14, 
					step = 1, 
					order = 2, 
				},
				timeformat = { 
					type = "toggle", 
					name = L["console.command.time.timeformat.name"], 
					desc = L["console.command.time.timeformat.description"], 
					get = "IsTimeFormatEnabled", 
					set = "ToggleTimeFormat", 
					order = 3, 
				}, 
			},
		},
		loot = {
			type = "group", 
			name = L["console.command.loot.name"], 
			desc = L["console.command.loot.description"], 
			order = 4, 
		    args = {
				minimum = {
					type = "text",
					name = L["console.command.loot.minimum.name"],
					desc = L["console.command.loot.minimum.description"], 
					usage = L["console.usage.item.quality"], 
					get = "GetMinimumLootQuality",
					set = "SetMinimumLootQuality", 
					validate = L.itemQuality, 
					order = 1, 
				},				
			}, 
		}, 
		export = {
			type = "group", 
			name = L["console.command.export.name"], 
			desc = L["console.command.export.description"], 
			order = 5, 
			args = {
				["format"] = {
					type = "text", 
					name = L["console.command.export.format.name"], 
					desc = L["console.command.export.format.description"], 
					usage = L["console.command.export.format.usage"], 
					get = "GetExportFormat", 
					set = "SetExportFormat", 
					validate = { ["EQdkp"] = L.exportFormat["EQdkp"] },
					order = 1, 
				}, 										
			}, 
		}, 
        ["debug"] = {
            type = "toggle", 
            name = L["console.command.debug.name"],
            desc = L["console.command.debug.description"],
            get = "IsDebugEnabled",
            set = "ToggleDebugEnabled", 
			order = 6, 
        }, 
		exclude = { 
			type = "group", 
			name = L["console.command.exclude.name"], 
			desc = L["console.command.exclude.description"], 			
			guiHidden = true, 
			order = 7, 
			args = { 
				add = {
					type = "text", 
					name = L["console.command.exclude.add.name"], 
					desc = L["console.command.exclude.add.description"], 
					usage = L["console.usage.exclude"], 
					message = L["console.command.exclude.add.set"],
					get = "GetExcludeText", 
					set = "AddToExclusionList", 
					validate = "ValidateAddToExclusionList", 
					order = 1, 
				}, 
				["remove"] = {
					type = "text", 
					name = L["console.command.exclude.remove.name"], 
					desc = L["console.command.exclude.remove.description"], 
					usage = L["console.usage.exclude"], 
					message = L["console.command.exclude.remove.set"],
					get = "GetExcludeText", 
					set = "RemoveFromExclusionList", 
					validate = "ValidateRemoveFromExclusionList", 
					order = 2, 
				}, 				
		        ["list"] = {
		            type = "execute", 
		            name = L["console.command.exclude.list.name"],
		            desc = L["console.command.exclude.list.description"],
					func = "DisplayExclusionList", 
					order = 3, 
		        },	
			}, 
		}, 
    },
}

-- Gets the options.
-- @return table Returns the options.
function HeadCount:getOptions()
	return options
end

-- default configuration options
local defaultOptions = { 
	raidListGroups = { ["1"] = true, ["2"] = true, ["3"] = true, ["4"] = true, ["5"] = true, ["6"] = false, ["7"] = false, ["8"] = false, }, 
	waitListGroups = { ["1"] = false, ["2"] = false, ["3"] = false, ["4"] = false, ["5"] = false, ["6"] = true, ["7"] = true, ["8"] = true, }, 
	timezone = -5, -- "-5 GMT, Eastern Standard Time" 
	isTimeFormatEnabled = true, 
	isRaidListTimeEnabled = true, 
	isWaitListTimeEnabled = false, 
	isOfflineTimeEnabled = false, 
	minimumLootQuality = L.itemQuality[5],
	exportFormat = L.exportFormat["EQdkp"], 
	isDebugEnabled = false, 
	raidListWrapper = nil, 
	exclusionListWrapper = nil, 
}

-- Gets the default options.
-- @return table Returns the default options.             
function HeadCount:getDefaultOptions()
	return defaultOptions
end

-- *** CONFIGURATION MENU ***
-- Opens the configuration menu
function HeadCount:ToggleUI() 
	HeadCount:raidUpdate()
	ToggleUserInterface()
end

-- *** OPTIONS API ***
function HeadCount:isGroupNumberAssigned(groupNumber)
	local isGroupNumberValid = false
	
	if (HeadCount:isRaidListGroup(groupNumber) or HeadCount:isWaitListGroup(groupNumber)) then
		-- the group number is in the raid list or wait list
		isGroupNumberValid = true
	end
	
	return isGroupNumberValid
end

-- Determines if a party group number is assigned to the raid list group.
-- @param groupNumber The group number.
-- @return boolean Returns true if the group number is assigned to the raid list, false otherwise.
function HeadCount:isRaidListGroup(groupNumber) 
	local isGroupNumberValid = false

	assert(type(groupNumber), "string", string.format(L["error.type.string"], L["product.name"], "isRaidListGroup"))
	
	local groupString = string.format("%d", groupNumber)
	if (self.db.profile.raidListGroups[groupString]) then
		isGroupNumberValid = true
	end
	
	return isGroupNumberValid
end

-- Determines if a party group number is assigned to the wait list group.
-- @return boolean Returns true if the group number is assigned to the wait list, false otherwise.
function HeadCount:isWaitListGroup(groupNumber) 
	local isGroupNumberValid = false

	assert(type(groupNumber), "string", string.format(L["error.type.string"], L["product.name"], "isWaitListGroup"))

	local groupString = string.format("%d", groupNumber)	
	if (self.db.profile.waitListGroups[groupString]) then
		isGroupNumberValid = true
	end
	
	return isGroupNumberValid
end

-- *** ATTENDANCE TRACKING ***
-- Gets the raid list groups table.
-- @return table Returns the raid list groups table.
function HeadCount:GetRaidListGroupsTable() 
	return self.db.profile.raidListGroups
end

-- Gets the raid list groups status.
-- @param key The raid list group key.
-- @return boolean Returns the raid list group status value.
function HeadCount:GetRaidListGroups(key) 
	return self.db.profile.raidListGroups[key]
end

-- Sets the raid list groups.
-- @param key The raid list group key.
-- @param value The raid list group status.
function HeadCount:SetRaidListGroups(key, value) 
	self.db.profile.raidListGroups[key] = value
	
	if (value) then
		self.db.profile.waitListGroups[key] = false
	end
	
	HeadCount:raidUpdate()
end

-- Gets the wait list groups status.
-- @return boolean Returns true if this group is a member of the wait list, false otherwise.
function HeadCount:GetWaitListGroups(key) 
	return self.db.profile.waitListGroups[key]
end

-- Sets the wait list groups.
-- @param key The wait list group key.
-- @param value The wait list group status.
function HeadCount:SetWaitListGroups(key, value) 
	self.db.profile.waitListGroups[key] = value
	
	if (value) then
		self.db.profile.raidListGroups[key] = false 
	end
	
	HeadCount:raidUpdate()
end

-- *** TIME ***
-- Gets the time zone difference (from UTC).
-- @return number Returns the time zone difference.
function HeadCount:GetTimezone() 
	return self.db.profile.timezone
end

-- Sets the time zone difference (from UTC).
-- @param timezone Sets the time zone difference.
function HeadCount:SetTimezone(timezone)
	self.db.profile.timezone = timezone
	HeadCountFrame_Update()	-- display change, update the UI
end

-- Gets the time format status
-- @return boolean Returns true if in 24-hour time format, false for 12-hour time format.
function HeadCount:IsTimeFormatEnabled() 
	return self.db.profile.isTimeFormatEnabled
end

-- Sets/toggles the time format status
function HeadCount:ToggleTimeFormat() 
	self.db.profile.isTimeFormatEnabled = not self.db.profile.isTimeFormatEnabled
	HeadCountFrame_Update()	-- display change, update the UI
end

-- Gets the raid list time status.
-- @return boolean Returns true if raid list time is enabled and false otherwise.
function HeadCount:IsRaidListTimeEnabled()
	return self.db.profile.isRaidListTimeEnabled
end

-- Sets/toggles the raid list time status.
function HeadCount:ToggleRaidListTime()
	self.db.profile.isRaidListTimeEnabled = not self.db.profile.isRaidListTimeEnabled
	HeadCountFrame_Update()
end

-- Gets the wait list time status.
-- @return boolean Returns true if wait list time is enabled and false otherwise.
function HeadCount:IsWaitListTimeEnabled()
	return self.db.profile.isWaitListTimeEnabled
end

-- Sets/toggles the wait list time status.
function HeadCount:ToggleWaitListTime()
	self.db.profile.isWaitListTimeEnabled = not self.db.profile.isWaitListTimeEnabled
	HeadCountFrame_Update()
end

-- Gets the offline time status.
-- @return boolean Returns true if offline time is enabled and false otherwise.
function HeadCount:IsOfflineTimeEnabled() 
	return self.db.profile.isOfflineTimeEnabled
end

-- Sets/toggles the offline time status.
function HeadCount:ToggleOfflineTime()
	self.db.profile.isOfflineTimeEnabled = not self.db.profile.isOfflineTimeEnabled
	HeadCountFrame_Update()
end

-- *** LOOT TRACKING ***
-- Gets the minimum loot tracking quality.
-- @return string Returns the minimum loot tracking quality.
function HeadCount:GetMinimumLootQuality()
	return self.db.profile.minimumLootQuality
end

-- Sets the minimum loot tracking quality.
-- @param minimumLootQuality The minimum loot quality.
function HeadCount:SetMinimumLootQuality(minimumLootQuality)
	self.db.profile.minimumLootQuality = self:convertStringToProperName(minimumLootQuality)
end

-- Gets the export format.
-- @return string Returns the export format.
function HeadCount:GetExportFormat()
	return self.db.profile.exportFormat
end

-- Sets the export format.
-- @param exportFormat The export format.
function HeadCount:SetExportFormat(exportFormat) 
	self.db.profile.exportFormat = exportFormat
end

-- *** DEBUG ***
-- Gets the debug mode status
-- @return boolean Returns true if the debug flag is enabled, false otherwise.
function HeadCount:IsDebugEnabled() 
	return self.db.profile.isDebugEnabled
end

-- Sets/toggles the debug mode status
function HeadCount:ToggleDebugEnabled() 
    self.db.profile.isDebugEnabled = not self.db.profile.isDebugEnabled
end

-- Gets the raid list wrapper.
-- @return table Returns the raid list wrapper.
function HeadCount:GetRaidListWrapper() 
	return self.db.profile.raidListWrapper
end

-- Sets the raid list wrapper.
-- @param raidListWrapper The raid list wrapper.
function HeadCount:SetRaidListWrapper(raidListWrapper) 
	self.db.profile.raidListWrapper = raidListWrapper
end

-- Gets the raid list wrapper.
-- @return table Returns the raid list wrapper.
function HeadCount:GetExclusionListWrapper() 
	return self.db.profile.exclusionListWrapper
end

-- Gets the add exclusion
-- @return string
function HeadCount:GetExcludeText() 
	local raidTracker = HeadCount:getRaidTracker()
	return string.format(L["info.exclude.number"], raidTracker:getNumberOfExcludedItems())
end 

-- Adds an item to the exclusion list
-- @param name The loot name or link
function HeadCount:AddToExclusionList(name) 	
	local args = AceLibrary("HeadCountLoot-1.0"):getArgs()
	local id = string.match(name, "item:(%d+):")	
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(id)

	args["itemId"] = id
	args["playerName"] = nil
	args["activityTime"] = nil
	args["quantity"] = 1
	args["zone"] = nil

	local loot = AceLibrary("HeadCountLoot-1.0"):new(args)	
	
	local raidTracker = HeadCount:getRaidTracker()		
	raidTracker:addExclusionItem(id, loot)	
	
	local exclusionListWrapper = raidTracker:getExclusionListWrapper()
	HeadCount:SetExclusionListWrapper(exclusionListWrapper)
end

-- Validates addition to the exclusion list.
-- @param name The loot link.
function HeadCount:ValidateAddToExclusionList(name) 
	local isValid = false
	local _, _, link = string.find(name, "(|c%x+|Hitem:[-%d:]+|h%[.-%]|h|r)")
	local id = string.match(name, "item:(%d+):")
	
	if ((link) and (id)) then 
		-- item passed in is a link		
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(id)
		if (itemName) then
			-- lookup for base item valid
			local raidTracker = HeadCount:getRaidTracker()
			if (raidTracker:isExclusionItemPresent(id)) then 
				HeadCount:LogWarning(string.format(L["error.exclude.duplicate"], L["product.name"], L["product.version.major"], L["product.version.minor"], itemLink))				
			else
				isValid = true
			end
		end
	end
	
	return isValid
end

-- Removes an item from the exclusion list.
-- @param link The loot link.
function HeadCount:RemoveFromExclusionList(link) 
	local args = AceLibrary("HeadCountLoot-1.0"):getArgs()
	local id = string.match(link, "item:(%d+):")	

	local raidTracker = HeadCount:getRaidTracker()			
	raidTracker:removeExclusionItem(id) 
	
	local exclusionListWrapper = raidTracker:getExclusionListWrapper()
	HeadCount:SetExclusionListWrapper(exclusionListWrapper)
end

-- Validates removal from the exclusion list.
-- @param link The loot link.
function HeadCount:ValidateRemoveFromExclusionList(link)
	local isValid = false
	local _, _, linkName = string.find(link, "(|c%x+|Hitem:[-%d:]+|h%[.-%]|h|r)")
	local id = string.match(link, "item:(%d+):")

	if ((linkName) and (id)) then 
		-- item passed in is a link
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(id)
		if (itemName) then 
			local raidTracker = HeadCount:getRaidTracker()
			if (raidTracker:isExclusionItemPresent(id)) then 
				isValid = true				
			else
				HeadCount:LogWarning(string.format(L["error.exclude.missing"], L["product.name"], L["product.version.major"], L["product.version.minor"], itemLink))				
			end			
		end
	end	
	
	return isValid
end

function HeadCount:DisplayExclusionList() 
	local raidTracker = HeadCount:getRaidTracker()
	local exclusionListWrapper = raidTracker:getExclusionListWrapper()
	exclusionListWrapper:list()
end

-- Sets the exclusion list wrapper.
-- @param exclusionListWrapper The exclusion list wrapper.
function HeadCount:SetExclusionListWrapper(exclusionListWrapper) 
	self.db.profile.exclusionListWrapper = exclusionListWrapper
end
