--[[
Project name: HeadCount
Developed by: seppyk (http://www.authority-km.com)
Website: http://www.wowace.com/wiki/HeadCount
Description: Manages tracking of raid attendance and more.
License: LGPL v2.1
File: Loot.lua
File description: Loot object
]]

local AceOO = AceLibrary("AceOO-2.0")
local L = AceLibrary("AceLocale-2.2"):new("HeadCount")

HeadCount.Loot = AceOO.Class()

HeadCount.Loot.prototype.itemId = nil
HeadCount.Loot.prototype.playerName = nil 
HeadCount.Loot.prototype.activityTime = nil
HeadCount.Loot.prototype.quantity = nil
HeadCount.Loot.prototype.zone = nil 

function HeadCount.Loot.prototype:init(args)
	self.class.super.prototype.init(self)

	self.type = "HeadCountLoot-1.0"
	
	self.itemId = args["itemId"]
	self.playerName = args["playerName"]	
	self.activityTime = args["activityTime"]
	self.quantity = args["quantity"]
	self.zone = args["zone"]
end

-- Gets the args structure
function HeadCount.Loot:getArgs() 
	local args = { 
		["itemId"] = nil, 
		["playerName"] = nil, 
		["activityTime"] = nil, 
		["quantity"] = nil, 
		["zone"] = nil,  
	}
	
	return args
end

-- Gets an item id from a link.
-- @param link The item link.
-- @return string Returns the item id or nil if none exists
function HeadCount.Loot:retrieveItemId(link)
	local startPoint
	local endPoint
	local itemId = nil
	
	if (link) then
		itemId = string.match(link, "item:(%d+):")
	end
	
	return itemId
end

-- Gets the item id.
-- @return string Returns the item id
function HeadCount.Loot.prototype:getItemId() 
	return self.itemId
end

-- Gets the player name.
-- @return string Returns the player name.
function HeadCount.Loot.prototype:getPlayerName() 
	return self.playerName 
end

-- Sets the player name.
-- @param playerName The player name.
function HeadCount.Loot.prototype:setPlayerName(playerName) 
	self.playerName = playerName
end

-- Gets the activity time.
-- @return object Returns the activity time.
function HeadCount.Loot.prototype:getActivityTime() 
	return self.activityTime
end

-- Gets the quantity.
-- @return number Returns the quantity.
function HeadCount.Loot.prototype:getQuantity() 
	return self.quantity
end

-- Gets the zone.
-- @return string Returns the zone
function HeadCount.Loot.prototype:getZone() 
	return self.zone
end

-- Gets the loot name.
-- @return string Returns the loot name.
function HeadCount.Loot.prototype:getName() 
	local itemName = GetItemInfo(self.itemId)

	return itemName
end

-- Gets the link.
-- @return string Returns the link.
function HeadCount.Loot.prototype:getLink() 
	local _, itemLink = GetItemInfo(self.itemId)	
	
	return itemLink	
end

-- Retrieve the item color
-- @return string Returns the item color.
function HeadCount.Loot.prototype:retrieveColor()
	local link = self:getLink()
	
	local _, _, color = string.find(link, "|c(%x+)|Hitem:[-%d:]+|h%[.-%]|h|r")
	
	return color
end

-- Gets the rarity.
-- @return number Returns the rarity.
function HeadCount.Loot.prototype:getRarity() 	
	local _, _, itemRarity = GetItemInfo(self.itemId)
	
	return itemRarity
end

-- Gets the item level.
-- @return number Returns the item level.
function HeadCount.Loot.prototype:getLevel() 
	local _, _, _, itemLevel = GetItemInfo(self.itemId)
	
	return itemLevel
end

-- Gets the item type.
-- @return string Returns the item type
function HeadCount.Loot.prototype:getItemType() 	
	local _, _, _, _, _, itemType = GetItemInfo(self.itemId)
	
	return itemType
end

-- Gets the item sub type.
-- @return string Returns the item sub type
function HeadCount.Loot.prototype:getItemSubType() 	
	local _, _, _, _, _, _, itemSubType = GetItemInfo(self.itemId)
	
	return itemSubType
end

-- Gets the texture.
-- @return string Returns the texture string
function HeadCount.Loot.prototype:getTexture() 
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, texture = GetItemInfo(self.itemId) 
	
	return texture
end

-- Retrieves the texture icon
-- @return string Returns the texture icon
function HeadCount.Loot.prototype:retrieveTextureIcon() 
	local texture = self:getTexture()
	local _, _, icon = string.find(texture, "^.*\\(.*)$")
	
	return icon
end


-- Serialization method.
function HeadCount.Loot.prototype:Serialize() 
	local s = { }
	
	for k,v in pairs(self) do
		if type(v) ~= "function" and type(v) ~= "userdata" and k ~= "header" and k ~= "btnframe" and k ~= "temp" and k ~= "theme" and k ~= "base" and k ~= "curState" then
			s[k] = v
		end
	end
	
	return s
end

-- Deserialization method.
function HeadCount.Loot:Deserialize(t) 
	return self:new(t)
end

-- To String
-- @return string Returns the string description for this object.
function HeadCount.Loot.prototype:ToString()
	return L["object.Loot"]
end

AceLibrary:Register(HeadCount.Loot, "HeadCountLoot-1.0", 1)